@@ -29,7 +29,7 @@ my $is_linux = ($^O =~ /linux/i) ? 1 : 0;
 my $is_osx = ($^O =~ /darwin/i) ? 1 : 0;
 my $is_gkfreebsd = ($^O =~ /gnukfreebsd/i) ? 1 : 0;
 
-# allow the user to override/specify the locations of OpenSSL and libssh2
+# allow the user to override/specify the locations of OpenSSL, libssh2 and libcurl
 our $opt = {};
 
 Getopt::Long::GetOptions(
@@ -38,10 +38,13 @@ Getopt::Long::GetOptions(
 	'with-openssl-libs=s@'   => \$opt->{'ssl'}->{'libs'},
 	'with-libssh2-include=s' => \$opt->{'ssh2'}->{'incdir'},
 	'with-libssh2-lib=s@'    => \$opt->{'ssh2'}->{'libs'},
+	'with-libcurl-include=s' => \$opt->{'curl'}->{'incdir'},
+	'with-libcurl-lib=s@'    => \$opt->{'curl'}->{'libs'},
 ) || die &usage();
 
 my $def = '';
 my $lib = '';
+my $otherldflags = '';
 my $inc = '';
 my $ccflags = '';
 
@@ -69,32 +72,41 @@ if (my $os_params = $os_specific{$^O}) {
 	}
 }
 
-my %library_tests = (
-	'ssl' => {
-		'lib'     => 'ssl',
-		'header'  => 'openssl/opensslconf.h',
-	},
-	'ssh2' => {
+my @library_tests = (
+	{
 		'lib'     => 'ssh2',
 		'libpath' => $ssh2_libpath,
 		'incpath' => $ssh2_incpath,
 		'header'  => 'libssh2.h',
-	}
+	},
+	{
+		'lib'     => 'ssl',
+		'header'  => 'openssl/opensslconf.h',
+	},
+	{
+		'lib'     => 'curl',
+		'header'  => 'curl/curl.h',
+	},
 );
 
 my %library_opts = (
 	'ssl' => {
-		'defines' => ' -DGIT_SSL',
+		'defines' => ' -DGIT_OPENSSL',
 		'libs'    => ' -lssl -lcrypto',
 	},
 	'ssh2' => {
 		'defines' => ' -DGIT_SSH',
 		'libs'    => ' -lssh2',
+	},
+	'curl' => {
+		'defines' => ' -DGIT_CURL',
+		'libs'    => ' -lcurl',
 	}
 );
 
 # check for optional libraries
-while (my ($library, $test) = each %library_tests) {
+foreach my $test (@library_tests) {
+	my $library = $test->{lib};
 	my $user_library_opt = $opt->{$library};
 	my $user_incpath = $user_library_opt->{'incdir'};
 	my $user_libs = $user_library_opt->{'libs'};
@@ -107,7 +119,7 @@ while (my ($library, $test) = each %library_tests) {
 			my ($link_dir, $link_lib) = (dirname($user_lib), basename($user_lib));
 
 			if (!$is_msvc) {
-				my @tokens = grep { $_ } split(/(lib|.)/, $link_lib);
+				my @tokens = grep { $_ } split(/(lib|\.)/, $link_lib);
 				shift @tokens if ($tokens[0] eq 'lib');
 				$link_lib = shift @tokens;
 			}
@@ -151,31 +163,38 @@ if ($is_solaris) {
 
 if ($is_gcc) {
 	# gcc-like compiler
-	$ccflags .= ' -Wall -Wno-unused-variable -Wdeclaration-after-statement';
+	$ccflags .= ' -Wall -Wno-unused-variable';
 
 	# clang compiler is pedantic!
 	if ($is_osx) {
-		$ccflags .= ' -Wno-deprecated-declarations -Wno-unused-const-variable -Wno-unused-function';
+		# clang masquerading as gcc
+		if ($Config{gccversion} =~ /LLVM/) {
+			$ccflags .= ' -Wno-deprecated-declarations -Wno-unused-const-variable -Wno-unused-function';
+		}
+
+		# Secure transport (HTTPS)
+		$def .= ' -DGIT_SECURE_TRANSPORT';
+		$otherldflags .= ' -framework CoreFoundation -framework Security';
 	}
 
 	if ($is_solaris) {
 		$ccflags .= ' -std=c99';
 	}
+
+	# building with a 32-bit perl on a 64-bit OS may require this (supported by cc and gcc-like compilers,
+	# excluding some ARM toolchains)
+	if ($Config{ptrsize} == 4 && $Config{archname} !~ /^armv/) {
+		$ccflags .= ' -m32';
+	}
 } elsif ($is_sunpro) {
-	# probably the SunPro compiler
+	# probably the SunPro compiler, (try to) enable C99 support
+	$ccflags .= ' -xc99=all,no_lib';
 	$def .= ' -D_STDC_C99';
 
 	$ccflags .= ' -errtags=yes -erroff=E_EMPTY_TRANSLATION_UNIT -erroff=E_ZERO_OR_NEGATIVE_SUBSCRIPT';
 	$ccflags .= ' -erroff=E_EMPTY_DECLARATION -erroff=E_STATEMENT_NOT_REACHED';
 }
 
-# building with a 32-bit perl on a 64-bit OS may require this (supported by cc and gcc-like compilers)
-if ($Config{gccversion} || $Config{gccversion}) {
-	if ($Config{ptrsize} == 4) {
-		$ccflags .= ' -m32';
-	}
-}
-
 # there are no atomic primitives for the Sun Pro compiler in libgit2, so even if pthreads is available
 # and perl has been built with threads support, libgit2 cannot use threads under said compiler
 if ($Config{usethreads} && !$is_sunpro) {
@@ -210,7 +229,8 @@ if ($is_windows || $is_solaris) {
 if ($is_windows) {
 	push @srcs, glob 'deps/libgit2/src/{win32,compat}/*.c';
 
-	$def .= ' -DWIN32 -DGIT_WIN32';
+	$def .= ' -DWIN32 -DGIT_WIN32 -DGIT_WINHTTP';
+	$lib .= ' -lwinhttp -lrpcrt4 -lcrypt32';
 
 	if ($is_msvc) {
 		# visual studio compiler
@@ -244,6 +264,26 @@ sub MY::c_o {
 	return $line;
 }
 
+sub MY::postamble {
+	if ($is_windows && !$is_msvc) {
+		my $def_file = "deps/libgit2/deps/winhttp/winhttp.def";
+		if ($Config{ptrsize} == 8) {
+			$def_file = "deps/libgit2/deps/winhttp/winhttp64.def";
+		}
+
+		my $line = qq{
+deps/libgit2/src/transports/winhttp.c: libwinhttp.a
+
+libwinhttp.a:
+	dlltool -d $def_file -k -D winhttp.dll -l libwinhttp.a
+		};
+
+		return $line;
+	}
+
+	return '';
+}
+
 # This Makefile.PL for {{ $distname }} was generated by Dist::Zilla.
 # Don't edit it but the dist.ini used to construct it.
 {{ $perl_prereq ? qq[BEGIN { require $perl_prereq; }] : ''; }}
@@ -260,6 +300,9 @@ $WriteMakefileArgs{LIBS}    .= $lib;
 $WriteMakefileArgs{INC}     .= $inc;
 $WriteMakefileArgs{CCFLAGS} .= $Config{ccflags} . ' '. $ccflags;
 $WriteMakefileArgs{OBJECT}  .= ' ' . join ' ', @objs;
+$WriteMakefileArgs{dynamic_lib} = {
+	OTHERLDFLAGS => $otherldflags
+};
 
 unless (eval { ExtUtils::MakeMaker->VERSION(6.56) }) {
 	my $br = delete $WriteMakefileArgs{BUILD_REQUIRES};
@@ -291,9 +334,18 @@ my @error_constants = (qw(
 	EUNMERGED
 	ENONFASTFORWARD
 	EINVALIDSPEC
-	EMERGECONFLICT
+	ECONFLICT
 	ELOCKED
 	EMODIFIED
+	EAUTH
+	ECERTIFICATE
+	EAPPLIED
+	EPEEL
+	EEOF
+	EINVALID
+	EUNCOMMITTED
+	EDIRECTORY
+	EMERGECONFLICT
 	PASSTHROUGH
 	ITEROVER
 
@@ -331,10 +383,19 @@ my @category_constants = (qw(
 	REVERT
 	CALLBACK
 	CHERRYPICK
+	DESCRIBE
+	REBASE
+	FILESYSTEM
 
 	INTERNAL
 ));
 
+my @packbuilder_constants = (qw(
+	ADDING_OBJECTS
+	DELTAFICATION
+));
+
+
 ExtUtils::Constant::WriteConstants(
 	NAME         => 'Git::Raw::Error',
 	NAMES        => [@error_constants],
@@ -355,6 +416,16 @@ ExtUtils::Constant::WriteConstants(
 	C_SUBNAME    => '_category_constant',
 );
 
+ExtUtils::Constant::WriteConstants(
+	NAME         => 'Git::Raw::Packbuilder',
+	NAMES        => [@packbuilder_constants],
+	DEFAULT_TYPE => 'IV',
+	C_FILE       => 'const-c-packbuilder.inc',
+	XS_FILE      => 'const-xs-packbuilder.inc',
+	XS_SUBNAME   => '_constant',
+	C_SUBNAME    => '_packbuilder_constant',
+);
+
 WriteMakefile(%WriteMakefileArgs);
 exit(0);
 
