@@ -1,7 +1,10 @@
 package Git::Raw::Blob;
-$Git::Raw::Blob::VERSION = '0.37'; # TRIAL
+$Git::Raw::Blob::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
+use overload
+	'""'       => sub { return $_[0] -> id },
+	fallback   => 1;
 
 use Git::Raw;
 
@@ -11,11 +14,11 @@ Git::Raw::Blob - Git blob class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Blob> represents a Git blob.
+A L<Git::Raw::Blob> represents a Git blob.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -29,7 +32,12 @@ Create a new blob from the given buffer.
 =head2 lookup( $repo, $id )
 
 Retrieve the blob corresponding to C<$id>. This function is pretty much the
-same as C<$repo-E<gt>lookup($id)> except that it only returns blobs.
+same as C<$repo-E<gt>lookup($id)> except that it only returns blobs. If the blob
+doesn't exist, this function wil return C<undef>.
+
+=head2 owner( )
+
+Retrieve the L<Git::Raw::Repository> owning the blob.
 
 =head2 content( )
 
@@ -43,6 +51,10 @@ Retrieve the size of the raw content of a blob.
 
 Return the raw ID (the SHA-1 hash) of the blob.
 
+=head2 is_binary( )
+
+Determine if the blob content is most certainly binary or not.
+
 =head2 is_blob( )
 
 Returns true.
