@@ -1,5 +1,5 @@
 package Git::Raw::Branch;
-$Git::Raw::Branch::VERSION = '0.37'; # TRIAL
+$Git::Raw::Branch::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Branch - Git branch class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 SYNOPSIS
 
@@ -47,21 +47,25 @@ Retrieve the branch corresponding to the given branch name.
 Rename the branch to C<$name>. Note that in order to get the updated branch
 object, an additional C<Git::Raw::Branch-E<gt>lookup()> is needed.
 
-=head2 upstream( )
+=head2 upstream( [$upstream] )
 
-Retrieve the reference supporting the remote tracking branch, given the local
-branch. If there is no corresponding supporting reference, this function will
-return C<undef>.
+Retrieve or set the reference supporting the remote tracking branch, given the
+local branch. If there is no corresponding supporting reference, this function
+will return C<undef>. C<$upstream> may either be a L<Git::Raw::Reference>, a
+string containing the upstream reference name or C<undef>. If C<$upstream> is
+set to C<undef>, the remote tracking branch association is removed.
 
 =head2 upstream_name( )
 
 Return the name of the reference supporting the remote tracking branch, given
-the the local branch.
+the the local branch. If there is no associated remote tracking branch, this
+function will return C<undef>.
 
 =head2 remote_name( )
 
 Return the name of remote that the remote tracking branch belongs to, given
-the local branch.
+the local branch. If there is no associated remote tracking branch, this
+function will return C<undef>.
 
 =head2 is_head( )
 
@@ -71,6 +75,8 @@ Check if the current local branch is pointed at by HEAD.
 
 Alessandro Ghedini <alexbio@cpan.org>
 
+Jacques Germishuys <jacquesg@striata.com>
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2012 Alessandro Ghedini.
