@@ -1,5 +1,5 @@
 package Git::Raw::Config;
-$Git::Raw::Config::VERSION = '0.37'; # TRIAL
+$Git::Raw::Config::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,11 +11,11 @@ Git::Raw::Config - Git config class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Config> represents a Git configuration file.
+A L<Git::Raw::Config> represents a Git configuration file.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -30,7 +30,7 @@ Create a new config object.
 
 Retrieve the global, XDG and system configuration and open them into a single
 prioritized config object that can be used when accessing default config data
-outside a repository
+outside a repository.
 
 =head2 add_file( $path, $level )
 
@@ -40,19 +40,19 @@ Add C<$path> to the config object with priority level C<$level>.
 
 Retrieve the value of the C<$name> configuration field of type boolean. If
 C<$value> is passed, the value of the configration will be updated and
-returned. If not C<$name> configuration is found, C<undef> is returned.
+returned. If C<$name> configuration is not found, C<undef> is returned.
 
 =head2 int( $name [, $value ] )
 
 Retrieve the value of the C<$name> configuration field of type integer. If
 C<$value> is passed, the value of the configration will be updated and
-returned. If not C<$name> configuration is found, C<undef> is returned.
+returned. If C<$name> configuration is not found, C<undef> is returned.
 
 =head2 str( $name [, $value ] )
 
 Retrieve the value of the C<$name> configuration field of type string. If
 C<$value> is passed, the value of the configration will be updated and
-returned. If not C<$name> configuration is found, C<undef> is returned.
+returned. If C<$name> configuration is not found, C<undef> is returned.
 
 =head2 foreach( $callback )
 
@@ -60,10 +60,6 @@ Run C<$callback> for every config entry. The callback receives the name of the
 config entry, its value and its priority level. A non-zero return value stops
 the loop.
 
-=head2 refresh( )
-
-Reload the config files from disk.
-
 =head2 delete( $name )
 
 Delete the variable C<$name> from the config object.
@@ -72,6 +68,8 @@ Delete the variable C<$name> from the config object.
 
 Alessandro Ghedini <alexbio@cpan.org>
 
+Jacques Germishuys <jacquesg@striata.com>
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2012 Alessandro Ghedini.
