@@ -1,5 +1,5 @@
 package Git::Raw::Diff;
-$Git::Raw::Diff::VERSION = '0.37'; # TRIAL
+$Git::Raw::Diff::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,11 +11,11 @@ Git::Raw::Diff - Git diff class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Diff> represents the diff between two entities.
+A L<Git::Raw::Diff> represents the diff between two entities.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -125,16 +125,16 @@ Maximum similarity sources to examine for a file (default 200).
 
 =head2 patches( )
 
-Return a list of C<Git::Raw::Patch> objects for the diff.
+Return a list of L<Git::Raw::Patch> objects for the diff.
 
 =head2 print( $format, $callback )
 
 Generate text output from the diff object. The C<$callback> will be called for
 each line of the diff with two arguments: the first one represents the type of
 the patch line (C<"ctx"> for context lines, C<"add"> for additions, C<"del">
-for deletions, C<"file"> for file headers, C<"hunk"> for hunk headers or
-C<"bin"> for binary data) and the second argument contains the content of the
-patch line.
+for deletions, C<"file"> for file headers, C<"hunk"> for hunk headers, C<"bin">
+for binary data or C<"noeol"> if both files have no LF at end) and the second
+argument contains the content of the patch line.
 
 The C<$format> can be one of the following:
 
@@ -164,7 +164,8 @@ Like C<git diff --name-status>.
 
 =head2 stats( )
 
-Accumlated diff statistics for all patches in the diff.
+Accumlated diff statistics for all patches in the diff. Returns a
+L<Git::Raw::Diff::Stats> object.
 
 =head1 AUTHOR
 
