@@ -1,5 +1,5 @@
 package Git::Raw::Index;
-$Git::Raw::Index::VERSION = '0.37'; # TRIAL
+$Git::Raw::Index::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,17 +11,21 @@ Git::Raw::Index - Git index class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Index> represents an index in a Git repository.
+A L<Git::Raw::Index> represents an index in a Git repository.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
 
 =head1 METHODS
 
+=head2 owner( )
+
+Retrieve the L<Git::Raw::Repository> owning the index.
+
 =head2 add( $entry )
 
 Add C<$entry> to the index. C<$entry> should either be the path of a file
@@ -66,6 +70,11 @@ be added to the index, C<E<gt>0> if it should be skipped or C<E<lt>0> to abort.
 
 =back
 
+=head2 find( $path )
+
+Find the first L<Git::Raw::Index::Entry> which point to given C<$path>. If an
+entry cannot be found, this function will return C<undef>.
+
 =head2 remove( $path )
 
 Remove C<$path> from the index.
@@ -98,7 +107,8 @@ Replace the index contente with C<$tree>.
 =head2 write_tree( [$repo] )
 
 Create a new tree from the index and write it to disk. C<$repo> optionally
-indicates which L<Git::Raw::Repository> the tree should be written to.
+indicates which L<Git::Raw::Repository> the tree should be written to. Returns
+a L<Git::Raw::Tree> object.
 
 =head2 checkout( [\%checkout_opts] )
 
@@ -110,9 +120,19 @@ C<%checkout_opts> values.
 
 Retrieve index entries. Returns a list of L<Git::Raw::Index::Entry> objects.
 
-=head2 remove_conflict( $file )
+=head2 add_conflict( $ancestor, $theirs, $ours)
 
-Remove C<$file> from the index.
+Add a new conflict entry. C<$ancestor>, C<$theirs> and C<$ours> should be
+L<Git::Raw::Index::Entry> objects.
+
+=head2 get_conflict( $path )
+
+Get the conflict entry for C<$path>. If C<$path> has no conflict entry,
+this function will return C<undef>.
+
+=head2 remove_conflict( $path )
+
+Remove C<$path> from the index.
 
 =head2 has_conflicts( )
 
@@ -125,7 +145,75 @@ Remove all conflicts in the index (entries with a stage greater than 0).
 =head2 conflicts( )
 
 Retrieve index entries that represent a conflict. Returns a list of
-L<Git::Raw::Index::Entry> objects.
+L<Git::Raw::Index::Conflict> objects.
+
+=head2 merge( $ancestor, $theirs, $ours, [\%merge_opts] )
+
+Merge two files as they exist in the index. C<$ancestor>, C<$theirs> and
+C<$ours> should be L<Git::Raw::Index::Entry> objects. Returns a
+L<Git::Raw::Merge::File::Result> object. Valid fields for the C<%merge_opts>
+hash are:
+
+=over 4
+
+=item * "our_label"
+
+The name of the "our" side of conflicts.
+
+=item * "their_label"
+
+The name of the "their" side of conflicts.
+
+=item * "ancestor_label"
+
+The name of the common ancestor side of conflicts.
+
+=item * "favor"
+
+Specify content automerging behaviour. Valid values are C<"ours">, C<"theirs">,
+and C<"union">.
+
+=item * "flags"
+
+Merge file flags. Valid values include:
+
+=over 8
+
+=item * "merge"
+
+Create standard conflicted merge file.
+
+=item * "diff3"
+
+Create diff3-style files.
+
+=item * "simplify_alnum"
+
+Condense non-alphanumeric regions for simplified diff file.
+
+=item * "ignore_whitespace"
+
+Ignore all whitespace.
+
+=item * "ignore_whitespace_change"
+
+Ignore changes in amount of whitespace.
+
+=item * "ignore_whitespace_eol"
+
+Ignore whitespace at end of line.
+
+=item * "patience"
+
+Use the C<"patience diff"> algorithm.
+
+=item * "minimal"
+
+Take extra time to find minimal diff.
+
+=back
+
+=back
 
 =head2 update_all( \%opts )
 
