@@ -1,5 +1,5 @@
 package Git::Raw::Index::Entry;
-$Git::Raw::Index::Entry::VERSION = '0.37'; # TRIAL
+$Git::Raw::Index::Entry::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,11 +11,11 @@ Git::Raw::Index::Entry - Git index entry class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Index::Entry> represents an index entry in a Git repository index.
+A L<Git::Raw::Index::Entry> represents an index entry in a Git repository index.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -26,6 +26,10 @@ B<WARNING>: The API of this module is unstable and may change without warning
 
 Retrieve the id of the index entry as a string.
 
+=head2 is_conflict( )
+
+Check if the entry has a non-zero stage entry.
+
 =head2 path( )
 
 Retrieve the path of the index entry.
@@ -38,6 +42,16 @@ Retrieve the size of the index entry.
 
 Retrieve the stage number for the index entry.
 
+=head2 blob( )
+
+Retrieve the blob for the the index entry. Returns a L<Git::Raw::Blob> object.
+
+=head2 clone( $path )
+
+Clone the entry, but use C<$path> as the entry's path. B<WARNING>: Use with
+caution. This is only useful for scenarios where a conflict entry should be
+added to the index where C<$path> should be used.
+
 =head1 AUTHOR
 
 Alessandro Ghedini <alexbio@cpan.org>
