@@ -1,5 +1,5 @@
 package Git::Raw::Patch;
-$Git::Raw::Patch::VERSION = '0.37'; # TRIAL
+$Git::Raw::Patch::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,11 +11,11 @@ Git::Raw::Patch - Git patch class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Patch> represents all the text diffs for a delta.
+A L<Git::Raw::Patch> represents all the text diffs for a delta.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -32,7 +32,7 @@ Get the number of hunks in the patch.
 
 =head2 hunks( [$index] )
 
-Returns a list of C<Git::Raw::Diff::Hunk> objects. If C<$index> is specified
+Returns a list of L<Git::Raw::Diff::Hunk> objects. If C<$index> is specified
 only the hunk at the specified index will be returned.
 
 =head2 line_stats( )
