@@ -1,5 +1,5 @@
 package Git::Raw::PathSpec;
-$Git::Raw::PathSpec::VERSION = '0.37'; # TRIAL
+$Git::Raw::PathSpec::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -9,11 +9,11 @@ Git::Raw::PathSpec - Git pathspec class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::PathSpec> represents a Git pathspec.
+A L<Git::Raw::PathSpec> represents a Git pathspec.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -27,10 +27,10 @@ Compile a new pathspec. C<@match> is the list of paths to match.
 =head2 match( $object [, \%options] )
 
 Math the pathspec against C<$object>. C<$object> could be a
-C<Git::Raw::Repository> (matches against the working directory),
-C<Git::Raw::Index> (matches against the index), C<Git::Raw::Tree> (matches
-against the tree) or a C<Git::Raw::Diff> (matches against the diff). Returns
-a C<Git::Raw::PathSpec::MatchList> object. Valid fields for C<%options> are:
+L<Git::Raw::Repository> (matches against the working directory),
+L<Git::Raw::Index> (matches against the index), L<Git::Raw::Tree> (matches
+against the tree) or a L<Git::Raw::Diff> (matches against the diff). Returns
+a L<Git::Raw::PathSpec::MatchList> object. Valid fields for C<%options> are:
 
 =over 4
 
