@@ -1,5 +1,5 @@
 package Git::Raw::Reference;
-$Git::Raw::Reference::VERSION = '0.37'; # TRIAL
+$Git::Raw::Reference::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,11 +11,11 @@ Git::Raw::Reference - Git reference class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Reference> represents a Git reference.
+A L<Git::Raw::Reference> represents a Git reference.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -65,9 +65,14 @@ This function returns either an object (L<Git::Raw::Blob>, L<Git::Raw::Commit>,
 L<Git::Raw::Tag> or L<Git::Raw::Tree>) for direct references, or another
 reference for symbolic references.
 
+=head2 peel( $type )
+
+Recursively peel the reference until an object of the specified C<$type> is
+found. Valid values for C<$type> include: C<"commit">, C<"tree"> or C<"tag">.
+
 =head2 reflog( )
 
-Retrieve the L<Git::Raw::Reflog> of the reference. Shortcur for
+Retrieve the L<Git::Raw::Reflog> of the reference. Shortcut for
 C<Git::Raw::Reflog-E<gt>open()>.
 
 =cut
@@ -88,16 +93,18 @@ Check if the reference is remote.
 
 =head2 is_tag( )
 
-Check if the reference lives in the L<"refs/tags"> namespace.
+Check if the reference lives in the C<refsE<sol>tags> namespace.
 
 =head2 is_note( )
 
-Check if the reference lives in the L<"refs/notes"> namespace.
+Check if the reference lives in the C<refsE<sol>notes> namespace.
 
 =head1 AUTHOR
 
 Alessandro Ghedini <alexbio@cpan.org>
 
+Jacques Germishuys <jacquesg@striata.com>
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2012 Alessandro Ghedini.
