@@ -1,5 +1,5 @@
 package Git::Raw::Repository;
-$Git::Raw::Repository::VERSION = '0.37'; # TRIAL
+$Git::Raw::Repository::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,24 +11,25 @@ Git::Raw::Repository - Git repository class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 SYNOPSIS
 
     use Git::Raw;
 
     # clone a Git repository
-    my $url  = 'git://github.com/ghedo/p5-Git-Raw.git';
-    my $repo = Git::Raw::Repository -> clone($url, 'p5-Git-Raw', {
-      'callbacks' => {
-        'transfer_progress' => sub {
-          my ($total_objects, $received_objects, $local_objects, $total_deltas,
-            $indexed_deltas, $received_bytes) = @_;
+    my $url  = 'git://github.com/jacquesg/p5-Git-Raw.git';
+    my $callbacks = {
+      'transfer_progress' => sub {
+        my ($total_objects, $received_objects, $local_objects, $total_deltas,
+          $indexed_deltas, $received_bytes) = @_;
 
-          print "Objects: $received_objects/$total_objects", "\n";
-          print "Received: ", int($received_bytes/1024), "KB", "\n";
-        }
+        print "Objects: $received_objects/$total_objects", "\n";
+        print "Received: ", int($received_bytes/1024), "KB", "\n";
       }
+    };
+    my $repo = Git::Raw::Repository -> clone($url, 'p5-Git-Raw', {}, {
+      'callbacks' => $callbacks
     });
 
     # print all the tags of the repository
@@ -38,7 +39,7 @@ version 0.37
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Repository> represents a Git repository.
+A L<Git::Raw::Repository> represents a Git repository.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -49,7 +50,7 @@ B<WARNING>: The API of this module is unstable and may change without warning
 
 Initialize a new repository at C<$path>.
 
-=head2 clone( $url, $path, \%opts )
+=head2 clone( $url, $path, \%opts, [\%fetch_opts, \%checkout_opts])
 
 Clone the repository at C<$url> to C<$path>. Valid fields for the C<%opts> hash
 are:
@@ -60,18 +61,10 @@ are:
 
 If true (default is false) create a bare repository.
 
-=item * "remote_name"
-
-The name to be given to the "origin" remote (default is "origin").
-
 =item * "checkout_branch"
 
 The name of the branch to checkout (default is to use the remote's HEAD).
 
-=item * "ignore_cert_errors"
-
-If true (default is false) ignore errors validating the remote host's certificate.
-
 =item * "disable_checkout"
 
 If true (default is false) files will not be checked out after the clone completes.
@@ -80,36 +73,23 @@ If true (default is false) files will not be checked out after the clone complet
 
 =over 8
 
-=item * "credentials"
+=item * "remote_create"
 
-The callback to be called any time authentication is required to connect to the
-remote repository. The callback receives a string containing the URL of the
-remote, and it must return a L<Git::Raw::Cred> object.
-
-=item * "sideband_progress"
-
-Textual progress from the remote. Text send over the progress side-band will be
-passed to this function (this is the 'counting objects' output). The callback
-receives a string containing progress information.
-
-=item * "transfer_progress"
-
-During the download of new data, this will be regularly called with the current
-count of progress done by the indexer. The callback receives the following
-integers: C<total_objects>, C<received_objects>, C<local_objects>,
-C<total_deltas>, C<indexed_deltas> and C<received_bytes>.
-
-=item * "update_tips"
-
-Each time a reference is updated locally, this function will be called with
-information about it. The callback receives a string containing the name of the
-reference that was updated, and the two OID's C<"a"> before and C<"b"> after the
-update.
+Remote customization callback. If a non-default remote is required, i.e. a remote
+with a remote name other than 'origin', this callback should be used. The callback
+receives a L<Git::Raw::Repository> object, a string containing the default name
+for the remote, typically 'origin', and a string containing the URL of the remote.
+This callbacks should return a L<Git::Raw::Remote> object. The returned object and
+the repository object passed to this callback is ephemeral. B<Note:> Do not take any
+references to it as it may be freed internally.
 
 =back
 
 =back
 
+See C<Git::Raw::Remote-E<gt>fetch()> for valid C<%fetch_opts> values and
+C<Git::Raw::Repository-E<gt>checkout()> for valid C<%checkout_opts> values.
+
 =head2 open( $path )
 
 Open the repository at C<$path>.
@@ -136,6 +116,13 @@ Retrieve the L<Git::Raw::Reference> pointed by the HEAD of the repository. If
 the L<Git::Raw::Reference> C<$new_head> is passed, the HEAD of the repository
 will be changed to point to it.
 
+=head2 detach_head( $commitish )
+
+Make the repository HEAD point directly to a commit. C<$commitish> should be
+peelable to a L<Git::Raw::Commit> object, that is, it should be a
+L<Git::Raw::Commit> or L<Git::Raw::Reference> object, or alternatively a commit
+id or commit id prefix.
+
 =head2 lookup( $id )
 
 Retrieve the object corresponding to C<$id>.
@@ -163,17 +150,15 @@ conflicts.
 Take any action to make the working directory match the target (pretty much the
 opposite of C<"none">.
 
+=item * "safe_create"
+
+Recreate missing files.
+
 =item * "safe"
 
 Make only modifications that will not lose changes (to be used in order to
 simulate C<git checkout>.
 
-=item * "safe_create"
-
-Like C<"safe">, but will also cause a file to be checked out if it is missing
-from the working directory even if it is not modified between the target and
-baseline (to be used in order to simulate C<git checkout-index> and C<git clone>).
-
 =item * "allow_conflicts"
 
 Apply safe updates even if there are conflicts.
@@ -194,6 +179,14 @@ Only update files that already exists (files won't be created not deleted).
 
 Do not write the updated files' info to the index.
 
+=item * "dont_remove_existing"
+
+Don not overwrite existing files or folders.
+
+=item * "dont_write_index"
+
+Prevent writing of the index upon completion.
+
 =item * "no_refresh"
 
 Do not reload the index and git attrs from disk before operations.
@@ -239,7 +232,7 @@ All of the above.
 
 =item * "callbacks"
 
-Hash containg progress and notification callbacks. Valid fields are:
+Hash containing progress and notification callbacks. Valid fields are:
 
 =over 8
 
@@ -248,14 +241,14 @@ Hash containg progress and notification callbacks. Valid fields are:
 This callback is called for each file matching one of the C<notify> options
 selected. It runs before modifying any files on disk. This callback should
 return a non-zero value should the checkout be cancelled.  The callback receives
-a string containing the path of the file C<path> and an array reference
-containing the reason C<why>.
+a string containing the path of the file C<$path> and an array reference
+containing the reason C<$why>.
 
 =item * "progress"
 
 The callback to be invoked as a file is checked out. The callback receives a
-string containing the path of the file C<path>, an integer C<completed_steps>
-and an integer C<total_steps>.
+string containing the path of the file C<$path>, an integer C<$completed_steps>
+and an integer C<$total_steps>.
 
 =back
 
@@ -264,6 +257,22 @@ and an integer C<total_steps>.
 An optional array representing the list of files thay should be checked out. If
 C<"paths"> is not specified, all files will be checked out (default).
 
+=item * "our_label"
+
+The name of the "our" side of conflicts.
+
+=item * "their_label"
+
+The name of the "their" side of conflicts.
+
+=item * "ancestor_label"
+
+The name of the common ancestor side of conflicts.
+
+=item * "target_directory"
+
+Alternative checkout path to the working directory.
+
 =back
 
 Example:
@@ -304,28 +313,143 @@ reset and the working directory will be replaced with the content of the index).
 =item * "paths"
 
 List of entries in the index to be updated from the target commit tree.  This is
-particularly useful to implement L<"git reset HEAD -- file file"> behaviour.
+particularly useful to implement C<"git reset HEAD -- file file"> behaviour.
 Note, if this parameter is specified, a value of C<"mixed"> will be used for
 C<"type"> (setting C<"type"> to C<"soft"> or C<"hard"> has no effect).
 
 =back
 
-=head2 status( [$file, $file, ...] )
+=head2 status( \%opts, [$file, $file, ...] )
 
 Retrieve the status of files in the index and/or working directory. This functions
 returns a hash reference with an entry for each C<$file>, or all files if no file
-parameters are provided. Each <$file> entry has a list of C<"flags">, which may
+parameters are provided. Each C<$file> entry has a list of C<"flags">, which may
 include: C<"index_new">, C<"index_modified">, C<"index_deleted">, C<"index_renamed">,
 C<"worktree_new">, C<"worktree_modified">, C<"worktree_deleted">,
-C<"worktree_renamed"> and C<"ignored">.
+C<"worktree_renamed">, C<"worktree_unreadable">, C<"conflicted"> and C<"ignored">.
 
 If C<$file> has been renamed in either the index or worktree or both, C<$file>
 will also have a corresponding entry C<"index"> and/or C<"worktree">, containing
 the previous filename C<"old_file">.
 
+Valid fields for the C<%opts> hash are:
+
+=over 4
+
+=item * "flags"
+
+Flags for the status. Valid values include:
+
+=over 8
+
+=item * "include_untracked"
+
+Callbacks should be made on untracked files. These will only be made if the
+workdir files are included in the C<$show> option.
+
+=item * "include_ignored"
+
+Callbacks should be made on ignored files. These will only be made if the
+ignored files get callbacks.
+
+=item * "include_unmodified"
+
+Include even unmodified files.
+
+=item * "exclude_submodules"
+
+Submodules should be skipped. This only applies if there are no pending
+typechanges to the submodule (either from or to another type).
+
+=item * "recurse_untracked_dirs"
+
+All files in untracked directories should be included. Normally if an entire
+directory is new, then just the top-level directory is included (with a
+trailing slash on the entry name). This flag includes all of the individual
+files in the directory instead.
+
+=item * "disable_pathspec_match"
+
+Each C<$file> specified should be treated as a literal path, and not as a
+pathspec pattern.
+
+=item * "recurse_ignored_dirs"
+
+The contents of ignored directories should be included in the status. This is
+like doing C<git ls-files -o -i --exclude-standard> with core git.
+
+=item * "renames_head_to_index"
+
+Rename detection should be processed between the head and the index.
+
+=item * "renames_index_to_workdir"
+
+Rename detection should be run between the index and the working directory.
+
+=item * "sort_case_sensitively"
+
+Override the native case sensitivity for the file system and forces the output
+to be in case-sensitive order.
+
+=item * "sort_case_insensitively"
+
+Override the native case sensitivity for the file system and forces the output
+to be in case-insensitive order.
+
+=item * "renames_from_rewrites"
+
+Rename detection should include rewritten files.
+
+=item * "no_refresh"
+
+Bypass the default status behavior of doing a "soft" index reload (i.e.
+reloading the index data if the file on disk has been modified outside
+C<Git::Raw>).
+
+=item * "update_index"
+
+Refresh the stat cache in the index for files that are unchanged but have out
+of date stat information in the index. It will result in less work being done
+on subsequent calls to C<status>. This is mutually exclusive with the
+C<"no_refresh"> option.
+
+=item * "include_unreadable"
+
+Include unreadable files.
+
+=item * "include_unreadable_as_untracked"
+
+Include unreadable files as untracked files.
+
+=back
+
+=item * "show"
+
+One of the following values (Defaults to C<index_and_worktree>):
+
+=over 8
+
+=item * "index_and_worktree"
+
+=item * "index"
+
+=item * "worktree"
+
+=back
+
+=back
+
 Example:
 
-    my $file_statuses = $repo -> status();
+    my $opts = {
+      'flags' => {
+        'include_untracked'        => 1,
+        'renames_head_to_index'    => 1,
+        'renames_index_to_workdir' => 1,
+      },
+      'show' => 'index_and_worktree'
+    };
+    my $file_statuses = $repo -> status($opts);
     while (my ($file, $status) = each %$file_statuses) {
       my $flags = $status -> {'flags'};
       print "File: $file: Status: ", join (' ', @$flags), "\n";
@@ -389,9 +513,9 @@ C<%checkout_opts> values.  Valid fields for C<%merge_opts> are
 
 =over 4
 
-=item * "flags"
+=item * "tree_flags"
 
-An array of flags for the tree, including:
+Merge tree flags. Valid values include:
 
 =over 8
 
@@ -401,6 +525,10 @@ Detect renames.
 
 =back
 
+=item * "file_flags"
+
+See C<Git::Raw::Index-E<gt>merge()> for options.
+
 =item * "favor"
 
 Specify content automerging behaviour. Valid values are C<"ours">, C<"theirs">,
@@ -421,17 +549,15 @@ Example:
 
     my $branch = Git::Raw::Branch -> lookup($repo, 'branch', 1);
     my $analysis = $repo -> merge_analysis($branch);
-    $repo -> merge($branch1,
-     {
-        'favor' => 'theirs'
-     },
-
-     {
+    my $merge_opts = {
+      'favor' => 'theirs'
+    };
+    my $checkout_opts = {
       'checkout_strategy' => {
         'force' => 1
       }
-     }
-    );
+    };
+    $repo -> merge($branch1, $merge_opts, $checkout_opts);
 
 =head2 ignore( $rules )
 
@@ -446,10 +572,10 @@ Checks the ignore rules to see if they would apply to the given file. This indic
 if the file would be ignored regardless of whether the file is already in the index
 or committed to the repository.
 
-=head2 diff( [\%opts] )
+=head2 diff( [\%diff_opts] )
 
 Compute the L<Git::Raw::Diff> between the repo's default index and another tree.
-Valid fields for the C<%opts> hash are:
+Valid fields for the C<%diff_opts> hash are:
 
 =over 4
 
@@ -545,6 +671,19 @@ Use the C<"patience diff"> algorithm.
 
 Take extra time to find minimal diff.
 
+=item * "show_binary"
+
+Include the necessary deflate / delta information so that C<git apply> can
+apply given diff information to binary files.
+
+=item * "force_text"
+
+Treat all files as text, disabling binary attributes and detection.
+
+=item * "force_binary"
+
+Treat all files as binary, disabling text diffs.
+
 =back
 
 =item * "prefix"
@@ -616,10 +755,10 @@ Create a new L<Git::Raw::Tag>. Shortcut for C<Git::Raw::Tag-E<gt>create()>.
 
 sub tag { return Git::Raw::Tag -> create(@_) }
 
-=head2 tags( )
+=head2 tags( [$type] )
 
-Retrieve the list of L<Git::Raw::Tag> objects representing the
-repository's annotated Git tags. Lightweight tags are not returned.
+Retrieve the list of annotated and/or lightweight tag objects. Possible values
+for C<$type> include C<"all">, C<"annotated"> or C<"lightweight">.
 
 =cut
 
@@ -630,12 +769,12 @@ sub tags {
 
 	Git::Raw::Tag -> foreach($self, sub {
 		push @tags, shift; 0
-	});
+	}, @_);
 
 	return @tags;
 }
 
-=head2 stash( $stasher, $msg )
+=head2 stash( $repo, $msg )
 
 Save the local modifications to a new stash. Shortcut for C<Git::Raw::Stash-E<gt>save()>.
 
@@ -670,7 +809,7 @@ working directory of the repository will be set to the directory.
 
 =head2 blame( $path )
 
-Retrieve blame information for C<$path>. Returns a C<Git::Raw::Blame> object.
+Retrieve blame information for C<$path>. Returns a L<Git::Raw::Blame> object.
 
 =head2 cherry_pick( $commit, [\%merge_opts, \%checkout_opts, $mainline] )
 
