@@ -1,5 +1,5 @@
 package Git::Raw::Tag;
-$Git::Raw::Tag::VERSION = '0.37'; # TRIAL
+$Git::Raw::Tag::VERSION = '0.54'; # TRIAL
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Tag - Git tag class
 
 =head1 VERSION
 
-version 0.37
+version 0.54
 
 =head1 SYNOPSIS
 
@@ -35,7 +35,7 @@ version 0.37
 
 =head1 DESCRIPTION
 
-A C<Git::Raw::Tag> represents an annotated Git tag.
+A L<Git::Raw::Tag> represents an annotated Git tag.
 
 B<WARNING>: The API of this module is unstable and may change without warning
 (any change will be appropriately documented in the changelog).
@@ -50,12 +50,20 @@ L<Git::Raw::Signature> representing the tagger and a target object.
 =head2 lookup( $repo, $id )
 
 Retrieve the tag corresponding to C<$id>. This function is pretty much the same
-as C<$repo-E<gt>lookup($id)> except that it only returns tags.
+as C<$repo-E<gt>lookup($id)> except that it only returns tags. If the tag
+doesn't exist, this function wil return C<undef>.
 
-=head2 foreach( $repo, $callback )
+=head2 owner( )
 
-Run C<$callback> for every tag in the repo. The callback receives a tag object.
-A non-zero return value stops the loop.
+Retrieve the L<Git::Raw::Repository> owning the tag.
+
+=head2 foreach( $repo, $callback, [$type] )
+
+Run C<$callback> for every tag in the repo. The callback receives a tag object,
+which will either a be a L<Git::Raw::Tag> object for annotated tags, or a
+L<Git::Raw::Reference> for lightweight tags. C<$type> may be C<"all">,
+C<"annotated"> or C<"lightweight">. If C<$type> is not specified or is C<undef>,
+all tags will be returned. A non-zero return value stops the loop.
 
 =head2 delete( )
 
@@ -75,7 +83,8 @@ Retrieve the message of the tag.
 
 =head2 tagger( )
 
-Retrieve the L<Git::Raw::Signature> representing the tag's tagger.
+Retrieve the L<Git::Raw::Signature> representing the tag's tagger. If there
+is no tagger, C<undef> will be returned.
 
 =head2 target( )
 
@@ -85,6 +94,8 @@ Retrieve the target object of the tag.
 
 Alessandro Ghedini <alexbio@cpan.org>
 
+Jacques Germishuys <jacquesg@striata.com>
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2012 Alessandro Ghedini.
